/*
  LED-strip-Lichtorgel (colour-organ) version 0.51 by Simon Brem (info 'at' simonbrem 'dot' de).
  For LED-strips with LPD8806-controller.
  My code is based on the following example-code and comes with the same license:
  

	Example of use of the FFT libray to compute FFT for a signal sampled through the ADC.
        Copyright (C) 2018 Enrique Condés and Ragnar Ranøyen Homb

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "arduinoFFT.h"        // libtary for fast-fourier-transformation
#include "LPD8806.h"           // LED strip with LPD8806-controller
#include "SPI.h"               // SPI-bus for LED strip

// #define DEBUG               // remove comment for debugging via serial-output, LED-strip will be off.
                               // If not in debug-mode, LED-strip must be connected, otherwhise program won't run

#define NUMBER_OF_LEDS 218     // The number of LEDs on your strip

int dataPin = 6;               // data and clock pins for LED-strip
int clockPin = 5;

// Set the first variable to the NUMBER of pixels. 32 = 32 pixels in a row
// The LED strips are 32 LEDs per meter but you can extend/cut the strip
LPD8806 strip = LPD8806(NUMBER_OF_LEDS, dataPin, clockPin);

#define AMPLIFY 4              // decrease value for higher response, increase value for lower respone
#define BASS 3                 // bass-band until this value
#define MIDS 7                 // mids-band until this value
#define TREBLE 15              // treble-band until this value (max. 31)

arduinoFFT FFT = arduinoFFT(); /* Create FFT object */

#define CHANNEL A0                      // use analog A0
const uint16_t samples = 64;            // This value MUST ALWAYS be a power of 2
const double samplingFrequency = 6000;  // Hz, must be less than 10000 due to ADC

unsigned int sampling_period_us;
unsigned long microseconds;

int bassdata;
int midsdata;
int trebledata;

double vReal[samples];        // These are the input and output vectors. Input vectors receive computed results from FFT
double vImag[samples];

#define SCL_INDEX 0x00
#define SCL_TIME 0x01
#define SCL_FREQUENCY 0x02
#define SCL_PLOT 0x03

void setup()
{
  sampling_period_us = round(1000000*(1.0/samplingFrequency));

  #ifdef DEBUG
    Serial.begin(115200);
    while(!Serial);
    Serial.println("Welcome to the debugging output of LED-strip-Lichtorgel");
  #endif

  #ifndef DEBUG
    strip.begin();        // start up the LED strip
    strip.show();         // update the strip, to start they are all 'off'
  #endif
}

void loop()
  {
  /*SAMPLING*/
  microseconds = micros();
  for(int i=0; i<samples; i++)
    {
    vReal[i] = analogRead(CHANNEL);
    vImag[i] = 0;
    while(micros() - microseconds < sampling_period_us){
      //empty loop
      }
    microseconds += sampling_period_us;
    }
  // Print the results of the sampling according to time 
  FFT.Windowing(vReal, samples, FFT_WIN_TYP_HAMMING, FFT_FORWARD);	// Weigh data 
  FFT.Compute(vReal, vImag, samples, FFT_FORWARD); // Compute FFT 
  FFT.ComplexToMagnitude(vReal, vImag, samples); // Compute magnitudes

  PrintVector(vReal, (samples >> 1), SCL_FREQUENCY);
  }

void PrintVector(double *vData, uint16_t bufferSize, uint8_t scaleType)
  {
  bassdata = 0;
  for (int band = 1; band < BASS; band++)       // leave out band 0
    bassdata =+ vData[band];                    // add bass-bands to bassdata
  bassdata = bassdata / BASS / AMPLIFY;         // calculate average value for bass-bands and amplify

  midsdata = 0;
  for (int band = (BASS+1); band < MIDS; band++)
    midsdata =+ vData[band];
  midsdata = midsdata / (MIDS-BASS) / AMPLIFY;

  trebledata = 0;
  for (int band = (MIDS+1); band < TREBLE; band++)
    trebledata =+ vData[band];
  trebledata = trebledata / (TREBLE-MIDS-BASS) / AMPLIFY;


  #ifdef DEBUG
    Serial.print("bass: ");
    Serial.print(bassdata, 4);
    Serial.print("   ");
    Serial.print("mids: ");
    Serial.print(midsdata, 4);
    Serial.print("   ");
    Serial.print("treble: ");
    Serial.println(trebledata, 4);
  #endif
    

  #ifndef DEBUG
    for (int i = 0; i < NUMBER_OF_LEDS; i++)
      strip.setPixelColor(i, bassdata, trebledata, midsdata);   // strip colours in  R B G  (!)
  
    strip.show();
  #endif
  }
